@@ -10,14 +10,14 @@
 %                            SSSSS   UUU   N   N                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write Sun Rasterfile Image Format.                  %
+%                    Read/Write Sun Rasterfile Image Format                   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,11 +40,15 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -54,6 +58,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/pixel-accessor.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -85,8 +90,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -137,7 +141,8 @@ static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
   const size_t length,unsigned char *pixels,size_t maxpixels)
 {
   register const unsigned char
-    *p, *l;
+    *l,
+    *p;
 
   register unsigned char
     *q;
@@ -214,7 +219,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    unsigned int
       magic,
       width,
       height,
@@ -231,8 +236,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     bit;
 
-  long
-    y;
+  MagickBooleanType
+    status;
 
   MagickSizeType
     number_pixels;
@@ -240,14 +245,12 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
-    x;
-
   register PixelPacket
     *q;
 
-  register long
-    i;
+  register ssize_t
+    i,
+    x;
 
   register unsigned char
     *p;
@@ -256,7 +259,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     length;
 
   ssize_t
-    count;
+    count,
+    y;
 
   SUNInfo
     sun_info;
@@ -266,9 +270,6 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *sun_pixels;
 
   unsigned int
-    status;
-
-  unsigned long
     bytes_per_line;
 
   /*
@@ -311,13 +312,18 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->rows=sun_info.height;
     if ((sun_info.depth == 0) || (sun_info.depth > 32))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    image->depth=sun_info.depth <= 8 ? sun_info.depth : MAGICKCORE_QUANTUM_DEPTH;
+    image->depth=sun_info.depth <= 8 ? sun_info.depth :
+      MAGICKCORE_QUANTUM_DEPTH;
     if (sun_info.depth < 24)
       {
+        size_t
+          one;
+
         image->storage_class=PseudoClass;
         image->colors=sun_info.maplength;
+        one=1;
         if (sun_info.maptype == RMT_NONE)
-          image->colors=1 << sun_info.depth;
+          image->colors=one << sun_info.depth;
         if (sun_info.maptype == RMT_EQUAL_RGB)
           image->colors=sun_info.maplength/3;
       }
@@ -350,13 +356,13 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (sun_colormap == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         count=ReadBlob(image,image->colors,sun_colormap);
-        for (i=0; i < (long) image->colors; i++)
+        for (i=0; i < (ssize_t) image->colors; i++)
           image->colormap[i].red=ScaleCharToQuantum(sun_colormap[i]);
         count=ReadBlob(image,image->colors,sun_colormap);
-        for (i=0; i < (long) image->colors; i++)
+        for (i=0; i < (ssize_t) image->colors; i++)
           image->colormap[i].green=ScaleCharToQuantum(sun_colormap[i]);
         count=ReadBlob(image,image->colors,sun_colormap);
-        for (i=0; i < (long) image->colors; i++)
+        for (i=0; i < (ssize_t) image->colors; i++)
           image->colormap[i].blue=ScaleCharToQuantum(sun_colormap[i]);
         sun_colormap=(unsigned char *) RelinquishMagickMemory(sun_colormap);
         break;
@@ -388,16 +394,11 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) CloseBlob(image);
         return(GetFirstImageInList(image));
       }
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     if ((sun_info.length*sizeof(*sun_data))/sizeof(*sun_data) !=
         sun_info.length || !sun_info.length)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     number_pixels=(MagickSizeType) image->columns*image->rows;
-    if ((sun_info.depth >= 8) &&
+    if ((sun_info.type != RT_ENCODED) && (sun_info.depth >= 8) &&
         ((number_pixels*((sun_info.depth+7)/8)) > sun_info.length))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     sun_data=(unsigned char *) AcquireQuantumMemory((size_t) sun_info.length,
@@ -411,7 +412,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     bytes_per_line=0;
     if (sun_info.type == RT_ENCODED)
       {
-        unsigned long
+        size_t
           height;
 
         /*
@@ -440,32 +441,34 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     p=sun_pixels;
     if (sun_info.depth == 1)
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-7); x+=8)
+        indexes=GetAuthenticIndexQueue(image);
+        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
-            indexes[x+7-bit]=(IndexPacket) ((*p) & (0x01 << bit) ? 0x00 : 0x01);
+            SetPixelIndex(indexes+x+7-bit,((*p) & (0x01 << bit) ?
+              0x00 : 0x01));
           p++;
         }
         if ((image->columns % 8) != 0)
           {
-            for (bit=7; bit >= (long) (8-(image->columns % 8)); bit--)
-              indexes[x+7-bit]=(IndexPacket)
-                ((*p) & (0x01 << bit) ? 0x00 : 0x01);
+            for (bit=7; bit >= (ssize_t) (8-(image->columns % 8)); bit--)
+              SetPixelIndex(indexes+x+7-bit,(*p) & (0x01 << bit) ?
+                0x00 : 0x01);
             p++;
           }
         if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
           p++;
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
           {
-            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+              image->rows);
             if (status == MagickFalse)
               break;
           }
@@ -478,21 +481,22 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
                (length > (bytes_per_line*image->rows))) ||
               ((sun_info.type != RT_ENCODED) && (length > sun_info.length)))
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
-            for (x=0; x < (long) image->columns; x++)
-              indexes[x]=(IndexPacket) (*p++);
+            indexes=GetAuthenticIndexQueue(image);
+            for (x=0; x < (ssize_t) image->columns; x++)
+              SetPixelIndex(indexes+x,*p++);
             if ((image->columns % 2) != 0)
               p++;
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
                 if (status == MagickFalse)
                   break;
               }
@@ -500,48 +504,58 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       else
         {
-          length=image->rows*((image->columns*(image->matte != MagickFalse ? 4 :
-            3))+image->columns % 2);
+          size_t
+            bytes_per_pixel;
+
+          bytes_per_pixel=3;
+          if (image->matte != MagickFalse)
+            bytes_per_pixel++;
+          length=image->rows*((bytes_per_line*image->columns)+
+            image->columns % 2);
           if (((sun_info.type == RT_ENCODED) &&
                (length > (bytes_per_line*image->rows))) ||
               ((sun_info.type != RT_ENCODED) && (length > sun_info.length)))
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
               if (image->matte != MagickFalse)
-                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
+                SetPixelAlpha(q,ScaleCharToQuantum(*p++));
               if (sun_info.type == RT_STANDARD)
                 {
-                  q->blue=ScaleCharToQuantum(*p++);
-                  q->green=ScaleCharToQuantum(*p++);
-                  q->red=ScaleCharToQuantum(*p++);
+                  SetPixelBlue(q,ScaleCharToQuantum(*p++));
+                  SetPixelGreen(q,ScaleCharToQuantum(*p++));
+                  SetPixelRed(q,ScaleCharToQuantum(*p++));
                 }
               else
                 {
-                  q->red=ScaleCharToQuantum(*p++);
-                  q->green=ScaleCharToQuantum(*p++);
-                  q->blue=ScaleCharToQuantum(*p++);
+                  SetPixelRed(q,ScaleCharToQuantum(*p++));
+                  SetPixelGreen(q,ScaleCharToQuantum(*p++));
+                  SetPixelBlue(q,ScaleCharToQuantum(*p++));
                 }
               if (image->colors != 0)
                 {
-                  q->red=image->colormap[(long) q->red].red;
-                  q->green=image->colormap[(long) q->green].green;
-                  q->blue=image->colormap[(long) q->blue].blue;
+                  SetPixelRed(q,image->colormap[(ssize_t)
+                    GetPixelRed(q)].red);
+                  SetPixelGreen(q,image->colormap[(ssize_t)
+                    GetPixelGreen(q)].green);
+                  SetPixelBlue(q,image->colormap[(ssize_t)
+                    GetPixelBlue(q)].blue);
                 }
               q++;
             }
-            if ((image->columns % 2) != 0)
+            if (((bytes_per_pixel*image->columns) % 2) != 0)
               p++;
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
                 if (status == MagickFalse)
                   break;
               }
@@ -605,10 +619,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterSUNImage method is:
 %
-%      unsigned long RegisterSUNImage(void)
+%      size_t RegisterSUNImage(void)
 %
 */
-ModuleExport unsigned long RegisterSUNImage(void)
+ModuleExport size_t RegisterSUNImage(void)
 {
   MagickInfo
     *entry;
@@ -688,7 +702,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    unsigned int
       magic,
       width,
       height,
@@ -699,9 +713,6 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
       maplength;
   } SUNInfo;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -711,17 +722,18 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
   MagickSizeType
     number_pixels;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
-  register long
+  register ssize_t
+    i,
     x;
 
-  register long
-    i;
+  ssize_t
+    y;
 
   SUNInfo
     sun_info;
@@ -744,13 +756,16 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SUN raster file header.
     */
-    if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
+    if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace);
     sun_info.magic=0x59a66a95;
-    sun_info.width=image->columns;
-    sun_info.height=image->rows;
-    sun_info.type=(unsigned long) 
-      (image->storage_class == DirectClass ? RT_FORMAT_RGB : RT_STANDARD);
+    if ((image->columns != (unsigned int) image->columns) ||
+        (image->rows != (unsigned int) image->rows))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    sun_info.width=(unsigned int) image->columns;
+    sun_info.height=(unsigned int) image->rows;
+    sun_info.type=(unsigned int) (image->storage_class == DirectClass ?
+      RT_FORMAT_RGB : RT_STANDARD);
     sun_info.maptype=RMT_NONE;
     sun_info.maplength=0;
     number_pixels=(MagickSizeType) image->columns*image->rows;
@@ -761,9 +776,10 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Full color SUN raster.
         */
-        sun_info.depth=(image->matte ? 32UL : 24UL);
-        sun_info.length=(unsigned long) ((image->matte ? 4 : 3)*number_pixels);
-        sun_info.length+=image->columns & 0x01 ? image->rows : 0;
+        sun_info.depth=(unsigned int) image->matte ? 32U : 24U;
+        sun_info.length=(unsigned int) ((image->matte ? 4 : 3)*number_pixels);
+        sun_info.length+=sun_info.length & 0x01 ? (unsigned int) image->rows :
+          0;
       }
     else
       if (IsMonochromeImage(image,&image->exception))
@@ -772,9 +788,10 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Monochrome SUN raster.
           */
           sun_info.depth=1;
-          sun_info.length=((image->columns+7) >> 3)*image->rows;
-          sun_info.length+=((image->columns/8)+(image->columns % 8 ? 1 : 0)) %
-            2 ? image->rows : 0;
+          sun_info.length=(unsigned int) (((image->columns+7) >> 3)*
+            image->rows);
+          sun_info.length+=(unsigned int) (((image->columns/8)+(image->columns %
+            8 ? 1 : 0)) % 2 ? image->rows : 0);
         }
       else
         {
@@ -782,10 +799,11 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Colormapped SUN raster.
           */
           sun_info.depth=8;
-          sun_info.length=(unsigned long) number_pixels;
-          sun_info.length+=image->columns & 0x01 ? image->rows : 0;
+          sun_info.length=(unsigned int) number_pixels;
+          sun_info.length+=(unsigned int) (image->columns & 0x01 ? image->rows :
+            0);
           sun_info.maptype=RMT_EQUAL_RGB;
-          sun_info.maplength=image->colors*3;
+          sun_info.maplength=(unsigned int) (3*image->colors);
         }
     /*
       Write SUN header.
@@ -809,6 +827,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
           *q;
 
         size_t
+          bytes_per_pixel,
           length;
 
         unsigned char
@@ -817,6 +836,9 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Allocate memory for pixels.
         */
+        bytes_per_pixel=3;
+        if (image->matte != MagickFalse)
+          bytes_per_pixel++;
         length=image->columns;
         pixels=(unsigned char *) AcquireQuantumMemory(length,4*sizeof(*pixels));
         if (pixels == (unsigned char *) NULL)
@@ -824,27 +846,28 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Convert DirectClass packet to SUN RGB pixel.
         */
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=pixels;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             if (image->matte != MagickFalse)
-              *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
-            *q++=ScaleQuantumToChar(p->red);
-            *q++=ScaleQuantumToChar(p->green);
-            *q++=ScaleQuantumToChar(p->blue);
+              *q++=ScaleQuantumToChar(GetPixelAlpha(p));
+            *q++=ScaleQuantumToChar(GetPixelRed(p));
+            *q++=ScaleQuantumToChar(GetPixelGreen(p));
+            *q++=ScaleQuantumToChar(GetPixelBlue(p));
             p++;
           }
-          if (image->columns & 0x01)
-            *q++=0;  /* pad scanline */
+          if (((bytes_per_pixel*image->columns) & 0x01) != 0)
+            *q++='\0';  /* pad scanline */
           (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -862,18 +885,18 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Convert PseudoClass image to a SUN monochrome image.
           */
           (void) SetImageType(image,BilevelType);
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=GetVirtualIndexQueue(image);
             bit=0;
             byte=0;
-            for (x=0; x < (long) image->columns; x++)
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
               byte<<=1;
-              if (PixelIntensity(p) < (MagickRealType) (QuantumRange/2.0))
+              if (GetPixelIntensity(image,p) < (QuantumRange/2.0))
                 byte|=0x01;
               bit++;
               if (bit == 8)
@@ -891,7 +914,8 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,SaveImageTag,y,image->rows);
+                status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
                 if (status == MagickFalse)
                   break;
               }
@@ -902,34 +926,36 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
           /*
             Dump colormap to file.
           */
-          for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,
-              ScaleQuantumToChar(image->colormap[i].red));
-          for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,
-              ScaleQuantumToChar(image->colormap[i].green));
-          for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,
-              ScaleQuantumToChar(image->colormap[i].blue));
+          for (i=0; i < (ssize_t) image->colors; i++)
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              image->colormap[i].red));
+          for (i=0; i < (ssize_t) image->colors; i++)
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              image->colormap[i].green));
+          for (i=0; i < (ssize_t) image->colors; i++)
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              image->colormap[i].blue));
           /*
             Convert PseudoClass packet to SUN colormapped pixel.
           */
-          for (y=0; y < (long) image->rows; y++)
+          for (y=0; y < (ssize_t) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
-            for (x=0; x < (long) image->columns; x++)
+            indexes=GetVirtualIndexQueue(image);
+            for (x=0; x < (ssize_t) image->columns; x++)
             {
-              (void) WriteBlobByte(image,(unsigned char) indexes[x]);
+              (void) WriteBlobByte(image,(unsigned char)
+                GetPixelIndex(indexes+x));
               p++;
             }
             if (image->columns & 0x01)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               {
-                status=SetImageProgress(image,SaveImageTag,y,image->rows);
+                status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                  image->rows);
                 if (status == MagickFalse)
                   break;
               }
