@@ -10,14 +10,14 @@
 %           A   A  R  R     T    IIIII  F      A   A  CCCCC    T              %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Artifact Methods                         %
+%                         MagickCore Artifact Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -248,6 +248,8 @@ MagickExport void DestroyImageArtifacts(Image *image)
 %
 %  GetImageArtifact() gets a value associated with an image artifact.
 %
+%  Note, the artifact is a constant.  Do not attempt to free it.
+%
 %  The format of the GetImageArtifact method is:
 %
 %      const char *GetImageArtifact(const Image *image,const char *key)
@@ -437,11 +439,15 @@ MagickExport MagickBooleanType SetImageArtifact(Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image->filename);
+
+  /* Create tree if needed */
   if (image->artifacts == (void *) NULL)
     image->artifacts=NewSplayTree(CompareSplayTreeString,
       RelinquishMagickMemory,RelinquishMagickMemory);
-  if ((value == (const char *) NULL) || (*value == '\0'))
+  /* Delete artifact if NULL --  empty string values are valid! */
+  if (value == (const char *) NULL)
     return(DeleteImageArtifact(image,artifact));
+  /* Add artifact to tree */
   status=AddValueToSplayTree((SplayTreeInfo *) image->artifacts,
     ConstantString(artifact),ConstantString(value));
   return(status);
