@@ -10,14 +10,14 @@
 %        EEEEE   X  X   CCCC  EEEEE  P       T    IIIII   OOO   N   N         %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Exception Methods                         %
+%                        MagickCore Exception Methods                         %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -156,7 +156,9 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
 
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  AcquireSemaphoreInfo(&exception->semaphore);
+  if (exception->exceptions  == (void *) NULL)
+    return;
+  LockSemaphoreInfo(exception->semaphore);
   p=(ExceptionInfo *) RemoveLastElementFromLinkedList((LinkedListInfo *)
     exception->exceptions);
   while (p != (ExceptionInfo *) NULL)
@@ -168,7 +170,7 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
   exception->severity=UndefinedException;
   exception->reason=(char *) NULL;
   exception->description=(char *) NULL;
-  RelinquishSemaphoreInfo(exception->semaphore);
+  UnlockSemaphoreInfo(exception->semaphore);
   errno=0;
 }
 
@@ -202,7 +204,9 @@ MagickExport void CatchException(ExceptionInfo *exception)
 
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  AcquireSemaphoreInfo(&exception->semaphore);
+  if (exception->exceptions  == (void *) NULL)
+    return;
+  LockSemaphoreInfo(exception->semaphore);
   ResetLinkedListIterator((LinkedListInfo *) exception->exceptions);
   p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
     exception->exceptions);
@@ -212,12 +216,12 @@ MagickExport void CatchException(ExceptionInfo *exception)
       MagickWarning(p->severity,p->reason,p->description);
     if ((p->severity >= ErrorException) && (p->severity < FatalErrorException))
       MagickError(p->severity,p->reason,p->description);
-    if (exception->severity >= FatalErrorException)
+    if (p->severity >= FatalErrorException)
       MagickFatalError(p->severity,p->reason,p->description);
     p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
       exception->exceptions);
   }
-  RelinquishSemaphoreInfo(exception->semaphore);
+  UnlockSemaphoreInfo(exception->semaphore);
   ClearMagickException(exception);
 }
 
@@ -254,10 +258,10 @@ static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
 {
   if (reason == (char *) NULL)
     return;
-  (void) fprintf(stderr,"%s: %s",GetClientName(),reason);
+  (void) FormatLocaleFile(stderr,"%s: %s",GetClientName(),reason);
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%s)",description);
-  (void) fprintf(stderr,".\n");
+    (void) FormatLocaleFile(stderr," (%s)",description);
+  (void) FormatLocaleFile(stderr,".\n");
   (void) fflush(stderr);
 }
 
@@ -284,24 +288,23 @@ static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
 %
 %    o severity: Specifies the numeric error category.
 %
-%    o reason: Specifies the reason to display before terminating the
-%      program.
+%    o reason: Specifies the reason to display before terminating the program.
 %
 %    o description: Specifies any description to the reason.
 %
 */
-static void DefaultFatalErrorHandler(const ExceptionType magick_unused(severity),
+static void DefaultFatalErrorHandler(const ExceptionType severity,
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
     return;
-  (void) fprintf(stderr,"%s: %s",GetClientName(),reason);
+  (void) FormatLocaleFile(stderr,"%s: %s",GetClientName(),reason);
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%s)",description);
-  (void) fprintf(stderr,".\n");
+    (void) FormatLocaleFile(stderr," (%s)",description);
+  (void) FormatLocaleFile(stderr,".\n");
   (void) fflush(stderr);
   MagickCoreTerminus();
-  exit(1);
+  exit((int) (severity-FatalErrorException)+1);
 }
 
 /*
@@ -319,12 +322,12 @@ static void DefaultFatalErrorHandler(const ExceptionType magick_unused(severity)
 %
 %  The format of the DefaultWarningHandler method is:
 %
-%      void DefaultWarningHandler(const ExceptionType warning,
+%      void DefaultWarningHandler(const ExceptionType severity,
 %        const char *reason,const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o warning: Specifies the numeric warning category.
+%    o severity: Specifies the numeric warning category.
 %
 %    o reason: Specifies the reason to display before terminating the
 %      program.
@@ -337,10 +340,10 @@ static void DefaultWarningHandler(const ExceptionType magick_unused(severity),
 {
   if (reason == (char *) NULL)
     return;
-  (void) fprintf(stderr,"%s: %s",GetClientName(),reason);
+  (void) FormatLocaleFile(stderr,"%s: %s",GetClientName(),reason);
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%s)",description);
-  (void) fprintf(stderr,".\n");
+    (void) FormatLocaleFile(stderr," (%s)",description);
+  (void) FormatLocaleFile(stderr,".\n");
   (void) fflush(stderr);
 }
 
@@ -373,7 +376,9 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  AcquireSemaphoreInfo(&exception->semaphore);
+  if (exception->semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&exception->semaphore);
+  LockSemaphoreInfo(exception->semaphore);
   exception->severity=UndefinedException;
   if (exception->exceptions != (void *) NULL)
     exception->exceptions=(void *) DestroyLinkedList((LinkedListInfo *)
@@ -381,7 +386,7 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
   relinquish=exception->relinquish;
   if (exception->relinquish != MagickFalse)
     exception->signature=(~MagickSignature);
-  RelinquishSemaphoreInfo(exception->semaphore);
+  UnlockSemaphoreInfo(exception->semaphore);
   DestroySemaphoreInfo(&exception->semaphore);
   if (relinquish != MagickFalse)
     exception=(ExceptionInfo *) RelinquishMagickMemory(exception);
@@ -416,6 +421,7 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
   (void) ResetMagickMemory(exception,0,sizeof(*exception));
   exception->severity=UndefinedException;
   exception->exceptions=(void *) NewLinkedList(0);
+  exception->semaphore=AllocateSemaphoreInfo();
   exception->signature=MagickSignature;
 }
 
@@ -435,16 +441,30 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 %
 %  The format of the GetExceptionMessage method is:
 %
-%      char *GetExceptionMessage(const int error_code)
+%      char *GetExceptionMessage(const int error)
 %
 %  A description of each parameter follows:
 %
-%    o error_code: the error code.
+%    o error: the error code.
 %
 */
-MagickExport char *GetExceptionMessage(const int error_code)
+MagickExport char *GetExceptionMessage(const int error)
 {
-  return(ConstantString(strerror(error_code)));
+  char
+    exception[MaxTextExtent];
+
+  *exception='\0';
+#if defined(MAGICKCORE_HAVE_STRERROR_R)
+#if !defined(MAGICKCORE_STRERROR_R_CHAR_P)
+  (void) strerror_r(error,exception,sizeof(exception));
+#else
+  (void) CopyMagickString(exception,strerror_r(error,exception,
+    sizeof(exception)),sizeof(exception));
+#endif
+#else
+  (void) CopyMagickString(exception,strerror(error),sizeof(exception));
+#endif
+  return(ConstantString(exception));
 }
 
 /*
@@ -489,6 +509,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case StreamWarning: return("Stream/Warning/");
     case CacheWarning: return("Cache/Warning/");
     case CoderWarning: return("Coder/Warning/");
+    case FilterWarning: return("Filter/Warning/");
     case ModuleWarning: return("Module/Warning/");
     case DrawWarning: return("Draw/Warning/");
     case ImageWarning: return("Image/Warning/");
@@ -497,6 +518,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case MonitorWarning: return("Monitor/Warning/");
     case RegistryWarning: return("Registry/Warning/");
     case ConfigureWarning: return("Configure/Warning/");
+    case PolicyWarning: return("Policy/Warning/");
     case ResourceLimitError: return("Resource/Limit/Error/");
     case TypeError: return("Type/Error/");
     case OptionError: return("Option/Error/");
@@ -508,6 +530,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case StreamError: return("Stream/Error/");
     case CacheError: return("Cache/Error/");
     case CoderError: return("Coder/Error/");
+    case FilterError: return("Filter/Error/");
     case ModuleError: return("Module/Error/");
     case DrawError: return("Draw/Error/");
     case ImageError: return("Image/Error/");
@@ -516,6 +539,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case MonitorError: return("Monitor/Error/");
     case RegistryError: return("Registry/Error/");
     case ConfigureError: return("Configure/Error/");
+    case PolicyError: return("Policy/Error/");
     case ResourceLimitFatalError: return("Resource/Limit/FatalError/");
     case TypeFatalError: return("Type/FatalError/");
     case OptionFatalError: return("Option/FatalError/");
@@ -527,6 +551,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case StreamFatalError: return("Stream/FatalError/");
     case CacheFatalError: return("Cache/FatalError/");
     case CoderFatalError: return("Coder/FatalError/");
+    case FilterFatalError: return("Filter/FatalError/");
     case ModuleFatalError: return("Module/FatalError/");
     case DrawFatalError: return("Draw/FatalError/");
     case ImageFatalError: return("Image/FatalError/");
@@ -535,6 +560,7 @@ static const char *ExceptionSeverityToTag(const ExceptionType severity)
     case MonitorFatalError: return("Monitor/FatalError/");
     case RegistryFatalError: return("Registry/FatalError/");
     case ConfigureFatalError: return("Configure/FatalError/");
+    case PolicyFatalError: return("Policy/FatalError/");
     default: break;
   }
   return("");
@@ -550,7 +576,7 @@ MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
     *locale_message;
 
   assert(tag != (const char *) NULL);
-  (void) FormatMagickString(message,MaxTextExtent,"Exception/%s%s",
+  (void) FormatLocaleString(message,MaxTextExtent,"Exception/%s%s",
     ExceptionSeverityToTag(severity),tag);
   locale_message=GetLocaleMessage(message);
   if (locale_message == (const char *) NULL)
@@ -594,7 +620,9 @@ MagickExport void InheritException(ExceptionInfo *exception,
   assert(exception->signature == MagickSignature);
   assert(relative != (ExceptionInfo *) NULL);
   assert(relative->signature == MagickSignature);
-  AcquireSemaphoreInfo(&exception->semaphore);
+  if (relative->exceptions == (void *) NULL)
+    return;
+  LockSemaphoreInfo(exception->semaphore);
   ResetLinkedListIterator((LinkedListInfo *) relative->exceptions);
   p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
     relative->exceptions);
@@ -604,7 +632,7 @@ MagickExport void InheritException(ExceptionInfo *exception,
     p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
       relative->exceptions);
   }
-  RelinquishSemaphoreInfo(exception->semaphore);
+  UnlockSemaphoreInfo(exception->semaphore);
 }
 
 /*
@@ -837,7 +865,7 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 %
 %    o severity: the severity of the exception.
 %
-%    o tag: the message tag.
+%    o reason: the reason for the exception.
 %
 %    o description: the exception description.
 %
@@ -890,7 +918,7 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
 %  The format of the ThrowMagickException method is:
 %
 %      MagickBooleanType ThrowFileException(ExceptionInfo *exception,
-%        const char *module,const char *function,const unsigned long line,
+%        const char *module,const char *function,const size_t line,
 %        const ExceptionType severity,const char *tag,const char *format,...)
 %
 %  A description of each parameter follows:
@@ -913,14 +941,17 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
 
 MagickExport MagickBooleanType ThrowMagickExceptionList(
   ExceptionInfo *exception,const char *module,const char *function,
-  const unsigned long line,const ExceptionType severity,const char *tag,
+  const size_t line,const ExceptionType severity,const char *tag,
   const char *format,va_list operands)
 {
   char
+    message[MaxTextExtent],
+    path[MaxTextExtent],
     reason[MaxTextExtent];
 
   const char
-    *locale;
+    *locale,
+    *type;
 
   int
     n;
@@ -945,12 +976,22 @@ MagickExport MagickBooleanType ThrowMagickExceptionList(
   if (n < 0)
     reason[MaxTextExtent-1]='\0';
   status=LogMagickEvent(ExceptionEvent,module,function,line,"%s",reason);
-  (void) ThrowException(exception,severity,reason,(char *) NULL);
+  GetPathComponent(module,TailPath,path);
+  type="undefined";
+  if ((severity >= WarningException) && (severity < ErrorException))
+    type="warning";
+  if ((severity >= ErrorException) && (severity < FatalErrorException))
+    type="error";
+  if (severity >= FatalErrorException)
+    type="fatal";
+  (void) FormatLocaleString(message,MaxTextExtent,"%s @ %s/%s/%s/%.20g",reason,
+    type,path,function,(double) line);
+  (void) ThrowException(exception,severity,message,(char *) NULL);
   return(status);
 }
 
 MagickExport MagickBooleanType ThrowMagickException(ExceptionInfo *exception,
-  const char *module,const char *function,const unsigned long line,
+  const char *module,const char *function,const size_t line,
   const ExceptionType severity,const char *tag,const char *format,...)
 {
   MagickBooleanType
