@@ -16,14 +16,14 @@
 %                           T    R  R   EEEEE  EEEEE                          %
 %                                                                             %
 %                                                                             %
-%            ImageMagick Self-adjusting Binary Search Tree Methods            %
+%             MagickCore Self-adjusting Binary Search Tree Methods            %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -97,7 +97,7 @@ struct _SplayTreeInfo
     *key,
     *next;
 
-  unsigned long
+  size_t
     nodes;
 
   MagickBooleanType
@@ -106,7 +106,7 @@ struct _SplayTreeInfo
   SemaphoreInfo
     *semaphore;
 
-  unsigned long
+  size_t
     signature;
 };
 
@@ -131,7 +131,9 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AddValueToSplayTree() adds a value to the splay-tree.
+%  AddValueToSplayTree() adds the given key and value to the splay-tree.
+%  Both key and value are used as is, without coping or cloning.  It returns
+%  MagickTrue on success, otherwise MagickFalse.
 %
 %  The format of the AddValueToSplayTree method is:
 %
@@ -156,7 +158,7 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_tree,
   register NodeInfo
     *node;
 
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,key);
   compare=0;
   if (splay_tree->root != (NodeInfo *) NULL)
@@ -168,24 +170,24 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_tree,
           ((splay_tree->root->key < key) ? -1 : 0);
       if (compare == 0)
         {
+          if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
+              (splay_tree->root->value != (void *) NULL))
+            splay_tree->root->value=splay_tree->relinquish_value(
+              splay_tree->root->value);
           if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
               (splay_tree->root->key != (void *) NULL))
             splay_tree->root->key=splay_tree->relinquish_key(
               splay_tree->root->key);
           splay_tree->root->key=(void *) key;
-          if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
-              (splay_tree->root->value != (void *) NULL))
-            splay_tree->root->value=splay_tree->relinquish_value(
-              splay_tree->root->value);
           splay_tree->root->value=(void *) value;
-          RelinquishSemaphoreInfo(splay_tree->semaphore);
+          UnlockSemaphoreInfo(splay_tree->semaphore);
           return(MagickTrue);
         }
     }
   node=(NodeInfo *) AcquireMagickMemory(sizeof(*node));
   if (node == (NodeInfo *) NULL)
     {
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return(MagickFalse);
     }
   node->key=(void *) key;
@@ -211,7 +213,7 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_tree,
   splay_tree->root=node;
   splay_tree->key=(void *) NULL;
   splay_tree->nodes++;
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(MagickTrue);
 }
 
@@ -240,13 +242,13 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_tree,
 %
 */
 
-static NodeInfo *LinkSplayTreeNodes(NodeInfo **nodes,const unsigned long low,
-  const unsigned long high)
+static NodeInfo *LinkSplayTreeNodes(NodeInfo **nodes,const size_t low,
+  const size_t high)
 {
   register NodeInfo
     *node;
 
-  unsigned long
+  size_t
     bisect;
 
   bisect=low+(high-low)/2;
@@ -325,9 +327,27 @@ static void BalanceSplayTree(SplayTreeInfo *splay_tree)
 %      whenever a value object is added to the splay-tree.
 %
 */
+
+static void *GetFirstSplayTreeNode(SplayTreeInfo *splay_tree)
+{
+  register NodeInfo
+    *node;
+
+  node=splay_tree->root;
+  if (splay_tree->root == (NodeInfo *) NULL)
+    return((NodeInfo *) NULL);
+  while (node->left != (NodeInfo *) NULL)
+    node=node->left;
+  return(node->key);
+}
+
 MagickExport SplayTreeInfo *CloneSplayTree(SplayTreeInfo *splay_tree,
   void *(*clone_key)(void *),void *(*clone_value)(void *))
 {
+  register NodeInfo
+    *next,
+    *node;
+
   SplayTreeInfo
     *clone_tree;
 
@@ -337,64 +357,28 @@ MagickExport SplayTreeInfo *CloneSplayTree(SplayTreeInfo *splay_tree,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   clone_tree=NewSplayTree(splay_tree->compare,splay_tree->relinquish_key,
     splay_tree->relinquish_value);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
-  if (splay_tree->root != (NodeInfo *) NULL)
+  LockSemaphoreInfo(splay_tree->semaphore);
+  if (splay_tree->root == (NodeInfo *) NULL)
     {
-      register NodeInfo
-        *active,
-        *next,
-        *node;
-
-      void
-        *key,
-        *value;
-
-      key=splay_tree->root->key;
-      if ((clone_key != (void *(*)(void *)) NULL) && (key != (void *) NULL))
-        key=clone_key(key);
-      value=splay_tree->root->value;
-      if ((clone_value != (void *(*)(void *)) NULL) && (value != (void *) NULL))
-        value=clone_value(value);
-      (void) AddValueToSplayTree(clone_tree,key,value);
-      for (node=splay_tree->root; node != (NodeInfo *) NULL; )
-      {
-        active=node;
-        for (node=(NodeInfo *) NULL; active != (NodeInfo *) NULL; )
-        {
-          next=(NodeInfo *) NULL;
-          if (active->left != (NodeInfo *) NULL)
-            {
-              next=node;
-              key=active->left->key;
-              if ((clone_key != (void *(*)(void *)) NULL) &&
-                  (key != (void *) NULL))
-                key=clone_key(key);
-              value=active->left->value;
-              if ((clone_value != (void *(*)(void *)) NULL) &&
-                  (value != (void *) NULL))
-                value=clone_value(value);
-              (void) AddValueToSplayTree(clone_tree,key,value);
-              node=active->left;
-            }
-          if (active->right != (NodeInfo *) NULL)
-            {
-              next=node;
-              key=active->right->key;
-              if ((clone_key != (void *(*)(void *)) NULL) &&
-                  (key != (void *) NULL))
-                key=clone_key(key);
-              value=active->right->value;
-              if ((clone_value != (void *(*)(void *)) NULL) &&
-                  (value != (void *) NULL))
-                value=clone_value(value);
-              (void) AddValueToSplayTree(clone_tree,key,value);
-              node=active->right;
-            }
-          active=next;
-        }
-      }
+      UnlockSemaphoreInfo(splay_tree->semaphore);
+      return(clone_tree);
     }
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  next=(NodeInfo *) GetFirstSplayTreeNode(splay_tree);
+  while (next != (NodeInfo *) NULL)
+  {
+    SplaySplayTree(splay_tree,next);
+    (void) AddValueToSplayTree(clone_tree,clone_key(splay_tree->root->key),
+      clone_value(splay_tree->root->value));
+    next=(NodeInfo *) NULL;
+    node=splay_tree->root->right;
+    if (node != (NodeInfo *) NULL)
+      {
+        while (node->left != (NodeInfo *) NULL)
+          node=node->left;
+        next=(NodeInfo *) node->key;
+      }
+  }
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(clone_tree);
 }
 
@@ -497,20 +481,6 @@ MagickExport int CompareSplayTreeStringInfo(const void *target,
 %    o value: the value.
 %
 */
-
-static void *GetFirstSplayTreeNode(SplayTreeInfo *splay_tree)
-{
-  register NodeInfo
-    *node;
-
-  node=splay_tree->root;
-  if (splay_tree->root == (NodeInfo *) NULL)
-    return((NodeInfo *) NULL);
-  while (node->left != (NodeInfo *) NULL)
-    node=node->left;
-  return(node->key);
-}
-
 MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
   SplayTreeInfo *splay_tree,const void *value)
 {
@@ -522,9 +492,12 @@ MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
   assert(splay_tree->signature == MagickSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  LockSemaphoreInfo(splay_tree->semaphore);
   if (splay_tree->root == (NodeInfo *) NULL)
-    return(MagickFalse);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+    {
+      UnlockSemaphoreInfo(splay_tree->semaphore);
+      return(MagickFalse);
+    }
   next=(NodeInfo *) GetFirstSplayTreeNode(splay_tree);
   while (next != (NodeInfo *) NULL)
   {
@@ -562,25 +535,25 @@ MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
             ((splay_tree->root->key < key) ? -1 : 0);
         if (compare != 0)
           {
-            RelinquishSemaphoreInfo(splay_tree->semaphore);
+            UnlockSemaphoreInfo(splay_tree->semaphore);
             return(MagickFalse);
           }
         left=splay_tree->root->left;
         right=splay_tree->root->right;
-        if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
-            (splay_tree->root->key != (void *) NULL))
-          splay_tree->root->key=splay_tree->relinquish_key(
-            splay_tree->root->key);
         if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
             (splay_tree->root->value != (void *) NULL))
           splay_tree->root->value=splay_tree->relinquish_value(
             splay_tree->root->value);
+        if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+            (splay_tree->root->key != (void *) NULL))
+          splay_tree->root->key=splay_tree->relinquish_key(
+            splay_tree->root->key);
         splay_tree->root=(NodeInfo *) RelinquishMagickMemory(splay_tree->root);
         splay_tree->nodes--;
         if (left == (NodeInfo *) NULL)
           {
             splay_tree->root=right;
-            RelinquishSemaphoreInfo(splay_tree->semaphore);
+            UnlockSemaphoreInfo(splay_tree->semaphore);
             return(MagickTrue);
           }
         splay_tree->root=left;
@@ -590,11 +563,11 @@ MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
               left=left->right;
             left->right=right;
           }
-        RelinquishSemaphoreInfo(splay_tree->semaphore);
+        UnlockSemaphoreInfo(splay_tree->semaphore);
         return(MagickTrue);
       }
   }
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(MagickFalse);
 }
 
@@ -609,7 +582,9 @@ MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DeleteNodeFromSplayTree() deletes a node from the splay-tree.
+%  DeleteNodeFromSplayTree() deletes a node from the splay-tree.  It returns
+%  MagickTrue if the option is found and successfully deleted from the
+%  splay-tree.
 %
 %  The format of the DeleteNodeFromSplayTree method is:
 %
@@ -639,7 +614,7 @@ MagickExport MagickBooleanType DeleteNodeFromSplayTree(
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_tree->root == (NodeInfo *) NULL)
     return(MagickFalse);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,key);
   splay_tree->key=(void *) NULL;
   if (splay_tree->compare != (int (*)(const void *,const void *)) NULL)
@@ -649,7 +624,7 @@ MagickExport MagickBooleanType DeleteNodeFromSplayTree(
       ((splay_tree->root->key < key) ? -1 : 0);
   if (compare != 0)
     {
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return(MagickFalse);
     }
   left=splay_tree->root->left;
@@ -666,7 +641,7 @@ MagickExport MagickBooleanType DeleteNodeFromSplayTree(
   if (left == (NodeInfo *) NULL)
     {
       splay_tree->root=right;
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return(MagickTrue);
     }
   splay_tree->root=left;
@@ -676,7 +651,7 @@ MagickExport MagickBooleanType DeleteNodeFromSplayTree(
         left=left->right;
       left->right=right;
     }
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(MagickTrue);
 }
 
@@ -711,17 +686,17 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
     *active,
     *pend;
 
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   if (splay_tree->root != (NodeInfo *) NULL)
     {
-      if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
-          (splay_tree->root->key != (void *) NULL))
-        splay_tree->root->key=splay_tree->relinquish_key(splay_tree->root->key);
-      splay_tree->root->key=(void *) NULL;
       if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
           (splay_tree->root->value != (void *) NULL))
         splay_tree->root->value=splay_tree->relinquish_value(
           splay_tree->root->value);
+      if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+          (splay_tree->root->key != (void *) NULL))
+        splay_tree->root->key=splay_tree->relinquish_key(splay_tree->root->key);
+      splay_tree->root->key=(void *) NULL;
       for (pend=splay_tree->root; pend != (NodeInfo *) NULL; )
       {
         active=pend;
@@ -729,27 +704,27 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
         {
           if (active->left != (NodeInfo *) NULL)
             {
-              if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
-                  (active->left->key != (void *) NULL))
-                active->left->key=splay_tree->relinquish_key(active->left->key);
-              active->left->key=(void *) pend;
               if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
                   (active->left->value != (void *) NULL))
                 active->left->value=splay_tree->relinquish_value(
                   active->left->value);
+              if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+                  (active->left->key != (void *) NULL))
+                active->left->key=splay_tree->relinquish_key(active->left->key);
+              active->left->key=(void *) pend;
               pend=active->left;
             }
           if (active->right != (NodeInfo *) NULL)
             {
+              if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
+                  (active->right->value != (void *) NULL))
+                active->right->value=splay_tree->relinquish_value(
+                  active->right->value);
               if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
                   (active->right->key != (void *) NULL))
                 active->right->key=splay_tree->relinquish_key(
                   active->right->key);
               active->right->key=(void *) pend;
-              if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
-                  (active->right->value != (void *) NULL))
-                active->right->value=splay_tree->relinquish_value(
-                  active->right->value);
               pend=active->right;
             }
           node=active;
@@ -759,7 +734,7 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
       }
     }
   splay_tree->signature=(~MagickSignature);
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   DestroySemaphoreInfo(&splay_tree->semaphore);
   splay_tree=(SplayTreeInfo *) RelinquishMagickMemory(splay_tree);
   return(splay_tree);
@@ -780,7 +755,7 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
 %
 %  The format of the GetNextKeyInSplayTree method is:
 %
-%      void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
+%      const void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
 %
 %  A description of each parameter follows:
 %
@@ -789,7 +764,7 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
 %    o key: the key.
 %
 */
-MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
+MagickExport const void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
 {
   register NodeInfo
     *node;
@@ -804,7 +779,7 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
   if ((splay_tree->root == (NodeInfo *) NULL) ||
       (splay_tree->next == (void *) NULL))
     return((void *) NULL);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,splay_tree->next);
   splay_tree->next=(void *) NULL;
   node=splay_tree->root->right;
@@ -815,7 +790,7 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
       splay_tree->next=node->key;
     }
   key=splay_tree->root->key;
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(key);
 }
 
@@ -834,7 +809,7 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
 %
 %  The format of the GetNextValueInSplayTree method is:
 %
-%      void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
+%      const void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
 %
 %  A description of each parameter follows:
 %
@@ -843,7 +818,7 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
 %    o key: the key.
 %
 */
-MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
+MagickExport const void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
 {
   register NodeInfo
     *node;
@@ -858,7 +833,7 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
   if ((splay_tree->root == (NodeInfo *) NULL) ||
       (splay_tree->next == (void *) NULL))
     return((void *) NULL);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,splay_tree->next);
   splay_tree->next=(void *) NULL;
   node=splay_tree->root->right;
@@ -869,7 +844,7 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
       splay_tree->next=node->key;
     }
   value=splay_tree->root->value;
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(value);
 }
 
@@ -886,9 +861,12 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
 %
 %  GetValueFromSplayTree() gets a value from the splay-tree by its key.
 %
+%  Note, the value is a constant.  Do not attempt to free it.
+%
 %  The format of the GetValueFromSplayTree method is:
 %
-%      void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,const void *key)
+%      const void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
+%        const void *key)
 %
 %  A description of each parameter follows:
 %
@@ -897,7 +875,7 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
 %    o key: the key.
 %
 */
-MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
+MagickExport const void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
   const void *key)
 {
   int
@@ -912,7 +890,7 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_tree->root == (NodeInfo *) NULL)
     return((void *) NULL);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,key);
   if (splay_tree->compare != (int (*)(const void *,const void *)) NULL)
     compare=splay_tree->compare(splay_tree->root->key,key);
@@ -921,11 +899,11 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
       ((splay_tree->root->key < key) ? -1 : 0);
   if (compare != 0)
     {
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return((void *) NULL);
     }
   value=splay_tree->root->value;
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(value);
 }
 
@@ -944,7 +922,7 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
 %
 %  The format of the GetNumberOfNodesInSplayTree method is:
 %
-%      unsigned long GetNumberOfNodesInSplayTree(
+%      size_t GetNumberOfNodesInSplayTree(
 %        const SplayTreeInfo *splay_tree)
 %
 %  A description of each parameter follows:
@@ -952,7 +930,7 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
 %    o splay_tree: the splay tree.
 %
 */
-MagickExport unsigned long GetNumberOfNodesInSplayTree(
+MagickExport size_t GetNumberOfNodesInSplayTree(
   const SplayTreeInfo *splay_tree)
 {
   assert(splay_tree != (SplayTreeInfo *) NULL);
@@ -1009,7 +987,7 @@ static int IterateOverSplayTree(SplayTreeInfo *splay_tree,
   NodeInfo
     **nodes;
 
-  register long
+  register ssize_t
     i;
 
   register NodeInfo
@@ -1137,7 +1115,7 @@ MagickExport SplayTreeInfo *NewSplayTree(
   splay_tree->next=(void *) NULL;
   splay_tree->nodes=0;
   splay_tree->debug=IsEventLogging();
-  splay_tree->semaphore=(SemaphoreInfo *) NULL;
+  splay_tree->semaphore=AllocateSemaphoreInfo();
   splay_tree->signature=MagickSignature;
   return(splay_tree);
 }
@@ -1185,7 +1163,7 @@ MagickExport void *RemoveNodeByValueFromSplayTree(SplayTreeInfo *splay_tree,
   key=(void *) NULL;
   if (splay_tree->root == (NodeInfo *) NULL)
     return(key);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   next=(NodeInfo *) GetFirstSplayTreeNode(splay_tree);
   while (next != (NodeInfo *) NULL)
   {
@@ -1220,7 +1198,7 @@ MagickExport void *RemoveNodeByValueFromSplayTree(SplayTreeInfo *splay_tree,
             ((splay_tree->root->key < key) ? -1 : 0);
         if (compare != 0)
           {
-            RelinquishSemaphoreInfo(splay_tree->semaphore);
+            UnlockSemaphoreInfo(splay_tree->semaphore);
             return(key);
           }
         left=splay_tree->root->left;
@@ -1234,7 +1212,7 @@ MagickExport void *RemoveNodeByValueFromSplayTree(SplayTreeInfo *splay_tree,
         if (left == (NodeInfo *) NULL)
           {
             splay_tree->root=right;
-            RelinquishSemaphoreInfo(splay_tree->semaphore);
+            UnlockSemaphoreInfo(splay_tree->semaphore);
             return(key);
           }
         splay_tree->root=left;
@@ -1244,11 +1222,11 @@ MagickExport void *RemoveNodeByValueFromSplayTree(SplayTreeInfo *splay_tree,
               left=left->right;
             left->right=right;
           }
-        RelinquishSemaphoreInfo(splay_tree->semaphore);
+        UnlockSemaphoreInfo(splay_tree->semaphore);
         return(key);
       }
   }
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(key);
 }
 
@@ -1297,7 +1275,7 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
   value=(void *) NULL;
   if (splay_tree->root == (NodeInfo *) NULL)
     return(value);
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   SplaySplayTree(splay_tree,key);
   splay_tree->key=(void *) NULL;
   if (splay_tree->compare != (int (*)(const void *,const void *)) NULL)
@@ -1307,7 +1285,7 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
       ((splay_tree->root->key < key) ? -1 : 0);
   if (compare != 0)
     {
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return(value);
     }
   left=splay_tree->root->left;
@@ -1321,7 +1299,7 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
   if (left == (NodeInfo *) NULL)
     {
       splay_tree->root=right;
-      RelinquishSemaphoreInfo(splay_tree->semaphore);
+      UnlockSemaphoreInfo(splay_tree->semaphore);
       return(value);
     }
   splay_tree->root=left;
@@ -1331,7 +1309,7 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
         left=left->right;
       left->right=right;
     }
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
   return(value);
 }
 
@@ -1340,6 +1318,97 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   R e s e t S p l a y T r e e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResetSplayTree() resets the splay-tree.  That is, it deletes all the nodes
+%  from the tree.
+%
+%  The format of the ResetSplayTree method is:
+%
+%      ResetSplayTree(SplayTreeInfo *splay_tree)
+%
+%  A description of each parameter follows:
+%
+%    o splay_tree: the splay tree.
+%
+*/
+MagickExport void ResetSplayTree(SplayTreeInfo *splay_tree)
+{
+  NodeInfo
+    *node;
+
+  register NodeInfo
+    *active,
+    *pend;
+
+  assert(splay_tree != (SplayTreeInfo *) NULL);
+  assert(splay_tree->signature == MagickSignature);
+  if (splay_tree->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  LockSemaphoreInfo(splay_tree->semaphore);
+  if (splay_tree->root != (NodeInfo *) NULL)
+    {
+      if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
+          (splay_tree->root->value != (void *) NULL))
+        splay_tree->root->value=splay_tree->relinquish_value(
+          splay_tree->root->value);
+      if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+          (splay_tree->root->key != (void *) NULL))
+        splay_tree->root->key=splay_tree->relinquish_key(splay_tree->root->key);
+      splay_tree->root->key=(void *) NULL;
+      for (pend=splay_tree->root; pend != (NodeInfo *) NULL; )
+      {
+        active=pend;
+        for (pend=(NodeInfo *) NULL; active != (NodeInfo *) NULL; )
+        {
+          if (active->left != (NodeInfo *) NULL)
+            {
+              if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
+                  (active->left->value != (void *) NULL))
+                active->left->value=splay_tree->relinquish_value(
+                  active->left->value);
+              if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+                  (active->left->key != (void *) NULL))
+                active->left->key=splay_tree->relinquish_key(active->left->key);
+              active->left->key=(void *) pend;
+              pend=active->left;
+            }
+          if (active->right != (NodeInfo *) NULL)
+            {
+              if ((splay_tree->relinquish_value != (void *(*)(void *)) NULL) &&
+                  (active->right->value != (void *) NULL))
+                active->right->value=splay_tree->relinquish_value(
+                  active->right->value);
+              if ((splay_tree->relinquish_key != (void *(*)(void *)) NULL) &&
+                  (active->right->key != (void *) NULL))
+                active->right->key=splay_tree->relinquish_key(
+                  active->right->key);
+              active->right->key=(void *) pend;
+              pend=active->right;
+            }
+          node=active;
+          active=(NodeInfo *) node->key;
+          node=(NodeInfo *) RelinquishMagickMemory(node);
+        }
+      }
+    }
+  splay_tree->root=(NodeInfo *) NULL;
+  splay_tree->key=(void *) NULL;
+  splay_tree->next=(void *) NULL;
+  splay_tree->nodes=0;
+  splay_tree->balance=MagickFalse;
+  UnlockSemaphoreInfo(splay_tree->semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e s e t S p l a y T r e e I t e r a t o r                               %
 %                                                                             %
 %                                                                             %
@@ -1365,9 +1434,9 @@ MagickExport void ResetSplayTreeIterator(SplayTreeInfo *splay_tree)
   assert(splay_tree->signature == MagickSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  AcquireSemaphoreInfo(&splay_tree->semaphore);
+  LockSemaphoreInfo(splay_tree->semaphore);
   splay_tree->next=GetFirstSplayTreeNode(splay_tree);
-  RelinquishSemaphoreInfo(splay_tree->semaphore);
+  UnlockSemaphoreInfo(splay_tree->semaphore);
 }
 
 /*
@@ -1402,7 +1471,7 @@ MagickExport void ResetSplayTreeIterator(SplayTreeInfo *splay_tree)
 %
 */
 
-static NodeInfo *Splay(SplayTreeInfo *splay_tree,const unsigned long depth,
+static NodeInfo *Splay(SplayTreeInfo *splay_tree,const size_t depth,
   const void *key,NodeInfo **node,NodeInfo **parent,NodeInfo **grandparent)
 {
   int
