@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -22,39 +22,11 @@
 extern "C" {
 #endif
 
-#if defined(__CYGWIN32__)
-#  if !defined(__CYGWIN__)
-#    define __CYGWIN__ __CYGWIN32__
-#  endif
-#endif
-
-#if defined(_WIN32) || defined(WIN32)
-#  if !defined(__WINDOWS__)
-#    if defined(_WIN32)
-#      define __WINDOWS__ _WIN32
-#    else
-#      if defined(WIN32)
-#        define __WINDOWS__ WIN32
-#      endif
-#    endif
-#  endif
-#endif
-
-#if defined(_WIN64) || defined(WIN64)
-#  if !defined(__WINDOWS__)
-#    if defined(_WIN64)
-#      define __WINDOWS__ _WIN64
-#    else
-#      if defined(WIN64)
-#        define __WINDOWS__ WIN64
-#      endif
-#    endif
-#  endif
-#endif
-
-#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
-# define MAGICKCORE_POSIX_SUPPORT
-#endif
+#if defined(WIN32) || defined(WIN64)
+#  define MAGICKCORE_WINDOWS_SUPPORT
+#else
+#  define MAGICKCORE_POSIX_SUPPORT
+#endif 
 
 #define MAGICKWAND_IMPLEMENTATION  1
 
@@ -74,108 +46,69 @@ extern "C" {
 #if defined(_magickcore_inline) && !defined(inline)
 # define inline _magickcore_inline
 #endif
+#if defined(_magickcore_restrict) && !defined(restrict)
+# define restrict  _magickcore_restrict
+#endif
 # if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
 # endif
-#if defined(_magickcore_restrict) && !defined(__restrict)
-# define __restrict _magickcore_restrict
-#endif
 #endif
 
 #if !defined(const)
 #  define STDC
 #endif
 
-#if defined(__BORLANDC__) && defined(_DLL)
-#  pragma message("BCBMagick lib DLL export interface")
-#  define _MAGICKDLL_
-#  define _MAGICKLIB_
-#endif
-
-#if defined(__WINDOWS__)
-# if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
-#  define _MAGICKDLL_
-# endif
-# if defined(_MAGICKDLL_)
-#  if defined(_VISUALC_)
-#   pragma warning( disable: 4273 )  /* Disable the dll linkage warnings */
-#  endif
-#  if !defined(_MAGICKLIB_)
-#   define WandExport  __declspec(dllimport)
-#   if defined(_VISUALC_)
-#    pragma message( "MagickWand lib DLL import interface" )
-#   endif
-#  else
-#   define WandExport  __declspec(dllexport)
-#   if defined(_VISUALC_)
-#    pragma message( "MagickWand lib DLL export interface" )
-#   endif
-#  endif
-# else
-#  define WandExport
-#  if defined(_VISUALC_)
-#   pragma message( "MagickWand lib static interface" )
-#  endif
-# endif
-
-# if defined(_DLL) && !defined(_LIB)
-#  define ModuleExport  __declspec(dllexport)
-#  if defined(_VISUALC_)
-#   pragma message( "MagickWand module DLL export interface" )
-#  endif
-# else
-#  define ModuleExport
-#  if defined(_VISUALC_)
-#   pragma message( "MagickWand module static interface" )
-#  endif
-
-# endif
-# define WandGlobal  __declspec(thread)
-# if defined(_VISUALC_)
-#  pragma warning(disable : 4018)
-#  pragma warning(disable : 4068)
-#  pragma warning(disable : 4244)
-#  pragma warning(disable : 4142)
-#  pragma warning(disable : 4800)
-#  pragma warning(disable : 4786)
-#  pragma warning(disable : 4996)
-# endif
-#else
-# define WandExport
-# define ModuleExport
-# define WandGlobal
-#endif
-
 #if defined(__cplusplus) || defined(c_plusplus)
 # define storage_class  c_class
 #else
 # define storage_class  class
 #endif
 
-#define WandSignature  0xabacadabUL
-#if !defined(MaxTextExtent)
-# define MaxTextExtent  4096
-#endif
-
 #include <stdarg.h>
 #include <stdio.h>
-#if defined(__WINDOWS__) && defined(_DEBUG)
+#if defined(MAGICKCORE_HAVE_SYS_STAT_H)
+# include <sys/stat.h>
+#endif
+#if defined(MAGICKCORE_STDC_HEADERS)
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# if defined(MAGICKCORE_HAVE_STDLIB_H)
+#  include <stdlib.h>
+# endif
+#endif
+#if defined(MAGICKCORE_HAVE_STRING_H)
+# if !defined(STDC_HEADERS) && defined(MAGICKCORE_HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#if defined(MAGICKCORE_HAVE_STRINGS_H)
+# include <strings.h>
+#endif
+#if defined(MAGICKCORE_HAVE_INTTYPES_H)
+# include <inttypes.h>
+#endif
+#if defined(MAGICKCORE_HAVE_STDINT_H)
+# include <stdint.h>
+#endif
+#if defined(MAGICKCORE_HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DEBUG)
 #define _CRTDBG_MAP_ALLOC
 #endif
-#include <stdlib.h>
-#if !defined(__WINDOWS__)
-# include <unistd.h>
-#else
+
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DEBUG)
+#define _CRTDBG_MAP_ALLOC
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 # include <direct.h>
 # if !defined(MAGICKCORE_HAVE_STRERROR)
 #  define HAVE_STRERROR
 # endif
 #endif
 
-#if defined(MAGICKCORE_HAVE_STRINGS_H)
-# include <strings.h>
-#endif
-#include <string.h>
 #include <ctype.h>
 #include <locale.h>
 #include <errno.h>
@@ -186,10 +119,13 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(MAGICKCORE_HAVE_PTHREAD)
+#if defined(MAGICKCORE_HAVE_XLOCALE_H)
+# include <xlocale.h>
+#endif
+#if defined(MAGICKCORE_THREAD_SUPPORT)
 # include <pthread.h>
-#elif defined(__WINDOWS__)
-#  define MAGICKORE_HAVE_WINTHREADS  1
+#elif defined(MAGICKCORE_WINDOWS_SUPPORT)
+#  define MAGICKCORE_HAVE_WINTHREADS  1
 #include <windows.h>
 #endif
 #if defined(MAGICKCORE_HAVE_SYS_SYSLIMITS_H)
@@ -199,9 +135,19 @@ extern "C" {
 # include <arm/limits.h>
 #endif
 
-#if defined(_OPENMP) && (_OPENMP >= 200203)
+#if defined(MAGICKCORE__OPENCL)
+#if defined(MAGICKCORE_HAVE_CL_CL_H)
+#  include <CL/cl.h>
+#endif
+#if defined(MAGICKCORE_HAVE_OPENCL_CL_H)
+#  include <OpenCL/cl.h>
+#endif
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
+#if defined(_OPENMP) && ((_OPENMP >= 200203) || defined(__OPENCC__))
 #  include <omp.h>
-#  define MAGICKCORE_OPENMP_SUPPORT 1
+#  define MAGICKCORE_OPENMP_SUPPORT  1
 #endif
 
 #if defined(MAGICKCORE_HAVE_PREAD) && defined(MAGICKCORE_HAVE_DECL_PREAD) && !MAGICKCORE_HAVE_DECL_PREAD
@@ -220,25 +166,7 @@ extern size_t strlcpy(char *,const char *,size_t);
 extern int vsnprintf(char *,size_t,const char *,va_list);
 #endif
 
-#if !defined(wand_attribute)
-#  if !defined(__GNUC__)
-#    define wand_attribute(x)  /* nothing */
-#  else
-#    define wand_attribute  __attribute__
-#  endif
-#endif
-
-#if !defined(wand_unused)
-#  if defined(__GNUC__)
-#     define wand_unused(x)  wand_unused_ ## x __attribute__((unused))
-#  elif defined(__LCLINT__)
-#    define wand_unused(x) /*@unused@*/ x
-#  else
-#    define wand_unused(x) x
-#  endif
-#endif
-
-#if defined(__WINDOWS__) || defined(MAGICKCORE_POSIX_SUPPORT)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(MAGICKCORE_POSIX_SUPPORT)
 # include <sys/types.h>
 # include <sys/stat.h>
 # if defined(MAGICKCORE_HAVE_FTIME)
@@ -271,7 +199,7 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
 # endif
 # include "wand/MagickWand.h"
-# if !defined(__WINDOWS__)
+# if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  include <sys/time.h>
 # if defined(MAGICKCORE_HAVE_SYS_TIMES_H)
 #  include <sys/times.h>
@@ -279,6 +207,9 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # if defined(MAGICKCORE_HAVE_SYS_RESOURCE_H)
 #  include <sys/resource.h>
 # endif
+# if defined(MAGICKCORE_HAVE_SYS_MMAN_H)
+#  include <sys/mman.h>
+# endif
 #endif
 #else
 # include <types.h>
@@ -290,18 +221,17 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  endif
 #  include <unix.h>
 # endif
-# include "wand/MagickWand.h"
 #endif
 
 #if defined(S_IRUSR) && defined(S_IWUSR)
 # define S_MODE (S_IRUSR | S_IWUSR)
-#elif defined (__WINDOWS__)
+#elif defined (MAGICKCORE_WINDOWS_SUPPORT)
 # define S_MODE (_S_IREAD | _S_IWRITE)
 #else
 # define S_MODE  0600
 #endif
 
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 # include "magick/nt-base.h"
 #endif
 #if defined(macintosh)
@@ -311,6 +241,8 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # include "magick/vms.h"
 #endif
 
+#include "wand/MagickWand.h"
+
 #undef HAVE_CONFIG_H
 #undef gamma
 #undef index
@@ -340,8 +272,6 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define IsBasenameSeparator(c) \
   (((c) == ']') || ((c) == ':') || ((c) == '/') ? MagickTrue : MagickFalse)
 #  define MAGICKCORE_LIBRARY_PATH  "sys$login:"
-#  define MAGICKCORE_CODER_PATH  "sys$login:"
-#  define MAGICKCORE_FILTER_PATH  "sys$login:"
 #  define MAGICKCORE_SHARE_PATH  "sys$login:"
 #  define X11_PREFERENCES_PATH  "decw$user_defaults:"
 #  define ProcessPendingEvents(text)
@@ -367,8 +297,6 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  ((c) == ':' ? MagickTrue : MagickFalse)
 #  define MAGICKCORE_LIBRARY_PATH  ""
-#  define MAGICKCORE_CODER_PATH  ""
-#  define MAGICKCORE_FILTER_PATH  ""
 #  define MAGICKCORE_SHARE_PATH  ""
 #  define X11_PREFERENCES_PATH  "~/."
 #  if defined(DISABLE_SIOUX)
@@ -384,7 +312,7 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
      SetWarningHandler(MACWarningHandler)
 #  endif
 # endif
-# if defined(__WINDOWS__)
+# if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  define DirectorySeparator  "\\"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
@@ -418,17 +346,13 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #define O_BINARY  0x00
 #endif
 
-/*
-  I/O defines.
-*/
-#if defined(__WINDOWS__) && !defined(Windows95) && !defined(__BORLANDC__)
-#define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
-#define MagickTell(file)  _telli64(file)
-#else
-#define MagickSeek(file,offset,whence)  lseek(file,offset,whence)
-#define MagickTell(file) tell(file)
+#if !defined(PATH_MAX)
+#define PATH_MAX  4096
 #endif
 
+/*
+  Exception defines.
+*/
 #define ThrowWandFatalException(severity,tag,context) \
 { \
   ExceptionInfo \
